<!DOCTYPE html>
<html>
    <head>
        <title>{% block title %}{% endblock %}</title>
        {% block extrahead %}{% endblock %}
    </head>
    <body>
        {% block content %}{% endblock %}
    </body>
</html>
