{% extends "base.html" %}
{% load static %}

{% block extrahead %}
<link rel="stylesheet" type="text/css" href="{% static 'css/gallery.css' %}">
{% endblock %}

{% block title %}{{ block.super }} - {{ photo.album.display_name }} - {{ photo.display_name }}{% endblock %}

{% block content %}
<h1>{{ photo.album.display_name }} - {{ photo.display_name }}</h1>
<div class="photo_nav">
{% if previous_photo %}
<a href="{{ previous_photo.get_absolute_url }}" class="previous">{{ previous_photo.display_name }}</a>
{% endif %}
<a href="{{ photo.album.get_absolute_url }}" class="up">{{ photo.album.display_name }}</a>
{% if next_photo %}
<a href="{{ next_photo.get_absolute_url }}" class="next">{{ next_photo.display_name }}</a>
{% endif %}
</div>

<div class="photo_detail">
<a href="{% url 'gallery:photo-original' pk=photo.pk %}"><img src="{% url 'gallery:photo-resized' preset='standard' pk=photo.pk %}" alt="{{ photo }}"></a>
</div>
{% endblock %}
