/* main page navigation */

.album_filter {
    text-align: right;
    margin-bottom: 1em;
}
.album_filter input {
    border: 1px solid transparent;
    border-bottom-color: #ccc;
}
.album_filter input[type="submit"] {
    background-color: #eee;
    border: 1px solid #eee;
    padding: 0 1em;
}
.album_filter input[type="submit"]:hover {
    border-color: #ccc;
}
.album_year_list,
.album_page_list,
.album_export {
    text-align: center;
}

/* previous | up | next navigation */

.photo_nav {
    position: relative;
    margin-bottom: 1.5em;
    padding: 0 35%;
    text-align: center;
}
.photo_nav .previous,
.photo_nav .next {
    position: absolute;
    top: 0;
    width: 35%;
}
.photo_nav .previous {
    left: 0;
    text-align: left;
}
.photo_nav .next {
    right: 0;
    text-align: right;
}
.photo_nav .up:before {
    content: "↑ ";
}
.photo_nav .up:after {
    content: " ↑";
}
.photo_nav .previous:before {
    content: "← ";
}
.photo_nav .next:after {
    content: " →";
}

/* titles in album lists */

.album_preview .album_count:before,
.album_preview .album_date:before {
    content: " - ";
}
.album_preview .album_category:before {
    content: "(";
}
.album_preview .album_category:after {
    content: ")";
}
.album_category {
    color: #999;
    font-style: italic;
}

/* layout of thumbnails and previews */

.photo_list,
.photo_detail {
    text-align: center;
}
.album_preview + .photo_list {
    overflow-x: auto;
    text-align: left;
    white-space: nowrap;
}
.photo_list img,
.photo_detail img {
    vertical-align: middle;
}
.photo_list a {
    box-shadow: 0.15em 0.15em 0.25em 0 #999;
    display: inline-block;
    margin: 0 1em 1em 0;
}
.photo_list a img {
    height: 8em;
    width: 8em;
}
.photo_detail a {
    max-width: 48em;
}
.photo_detail a img {
    max-height: 48em;
    max-width: 100%;
}
