{% extends "admin/base_site.html" %}
{% load i18n %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; <a href="{% url 'admin:app_list' app_label='gallery' %}">Gallery</a>
&rsaquo; {% trans 'Scan' %}
</div>
{% endblock %}

{% block content %}
<p>{% trans 'Click this button to re-scan your photo directory.' %}</p>
<form action="{% url 'admin:gallery_scan_photos' %}" method="POST">
    {% csrf_token %}
    <input type="submit" value="{% trans 'Scan photos' %}" />
</form>
{% endblock %}
