# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gallery',
 'gallery.management',
 'gallery.management.commands',
 'gallery.migrations',
 'gallery.resizers']

package_data = \
{'': ['*'],
 'gallery': ['locale/fr/LC_MESSAGES/*',
             'static/css/*',
             'templates/admin/gallery/*',
             'templates/gallery/*',
             'test_templates/*']}

install_requires = \
['django>=3.2']

setup_kwargs = {
    'name': 'myks-gallery',
    'version': '0.9',
    'description': 'Photo gallery with granular access control',
    'long_description': 'mYk\'s gallery\n#############\n\nIntroduction\n============\n\n`myks-gallery`_ is a simple photo gallery with granular access control.\n\nIt powers my `humble photo gallery`_, allowing me to:\n\n- access my entire photo collection privately,\n- share some albums with family or friends,\n- make some albums public.\n\n.. _myks-gallery: https://github.com/aaugustin/myks-gallery\n.. _humble photo gallery: https://myks.org/photos/\n\nUse case\n========\n\nRather than use a photo manager, I just create a new directory for each event\nand put my photos inside. I include the date of the event in the name of the\ndirectory and I rename photos based on their date and time. Then I regularly\nsynchronize my collection to a remote storage. I serve my gallery from there.\n\nIf you have a similar workflow, you may find myks-gallery useful.\n\nWhenever I upload new photos, I re-scan the collection with ``django-admin\nscanphotos`` or with the button in the admin. myks-gallery detects new albums\nand photos. Then I define users, groups and access policies in the admin.\n\nAlbum access policies control the visibility of albums. Most often, you\'ll\nenable the "photos inherit album access policy" option. If you need more\ncontrol, for instance to share only a subset of an album, you can disable this\noption and use photo access policies. You still need to define an album access\npolicy and it should be a superset of the photo access policies.\n\nObviously, requiring usernames and passwords doesn\'t work well for sharing\nphotos with relatives. You might want to use django-sesame_.\n\n.. _django-sesame: https://github.com/aaugustin/django-sesame\n\nSetup\n=====\n\nmyks-gallery is a pluggable Django application. It requires:\n\n* Django ≥ 3.2 (LTS)\n* Python ≥ 3.6\n\nArchitecture\n------------\n\nmyks-gallery requires two storage areas:\n\n- The first one contains the original photos. It\'s a read-only reference. You\n  can upload photos with `aws s3 sync`_, `gsutil rsync`_, etc.\n- The second one contains resized versions and ZIP archives of album exports.\n  It\'s a cache. You can set up expiry policies and clear it without affecting\n  the gallery, aside from the cost of rescaling images again.\n\nmyks-gallery accesses them through Django\'s `file storage API`_, meaning that\nyou can use any storage for which a Django storage backend exists. You should\nuse a third-party storage backend if you\'re storing files in a cloud service.\n\n.. _aws s3 sync: https://docs.aws.amazon.com/cli/latest/reference/s3/sync.html\n.. _gsutil rsync: https://cloud.google.com/storage/docs/gsutil/commands/rsync\n.. _file storage API: https://docs.djangoproject.com/en/stable/ref/files/storage/\n\nInstallation guide\n------------------\n\nThis application isn\'t exactly plug\'n\'play. There are many moving pieces.\nHere\'s the general process for integrating myks-gallery into an existing\nwebsite:\n\n1.  Download and install the package from PyPI::\n\n        $ pip install myks-gallery Pillow\n\n2.  Add ``gallery.apps.GalleryConfig`` to ``INSTALLED_APPS``::\n\n        INSTALLED_APPS += [\'gallery.apps.GalleryConfig\']\n\n3.  Configure the settings — see below for the list.\n\n4.  Add the application to your URLconf with the ``gallery`` application\n    namespace::\n\n        urlpatterns += [\n            path(\'gallery/\', include(\'gallery.urls\', namespace=\'gallery\')),\n        ]\n\n5.  Create a suitable ``base.html`` template. It must provide three blocks:\n    ``title``, ``extrahead``, ``content``, as shown in this `example`_.\n\n6.  Scan your photos with the "Scan photos" button in the admin or the\n    ``scanphotos`` management command and define access policies.\n\nThe source_ contains a sample application in the ``example`` directory. It can\nhelp you see how everything fits together. See below for how to run it.\n\n.. _example: https://github.com/aaugustin/myks-gallery/blob/master/example/example/templates/base.html\n.. _X-accel: http://wiki.nginx.org/X-accel\n.. _mod_xsendfile: https://tn123.org/mod_xsendfile/\n.. _source: https://github.com/aaugustin/myks-gallery\n\nAccess control\n--------------\n\nmyks-gallery provides two levels of access control: by album and by photo.\n\nBy default, albums and photos aren\'t visible by anyone, except users with the\n"Can see all photos" permission, including superusers who have it implicitly.\n\nTo make them visible, you must define an access policy. You have two options:\npublic access or access restricted to select users or groups.\n\nAccess policies for albums are configured explicitly in the admin.\n\nIn most cases, you will enable the "Photos inherit album access policy"\noption, so that the access policy also applies to all photos in the album.\n\nAccess policies for photos may also be configured for granular control.\n\nFor example, if you want to publish just a few photos in an album, make these\nphotos public, make the album public, but don\'t enable "Photos inherit album\naccess policy". Other photos in the album won\'t be visible.\n\nAnother example, if you want to share an album privately except for a few\nphotos, set an empty access policy on these photos (e.g. by adding then\nremoving yourself), then allow some groups or users to view the album.\n\nPermissions\n-----------\n\nmyks-gallery defines two permissions for ``django.contrib.auth``:\n\n- "Can scan the photos directory" allows using the "Scan photos" button in the\n  admin.\n- "Can see all photos" allows seeing all albums and all photos regardless of\n  access policies.\n\nSettings\n--------\n\n``GALLERY_PHOTO_STORAGE``\n.........................\n\nDefault: *not defined*\n\nDotted Python path to the Django storage class for the original photos. It\nmust be readable by the application server but should not be writable.\n\nWhile ``GALLERY_PHOTO_STORAGE`` behaves like Django\'s ``DEFAULT_FILE_STORAGE``\nsetting, you\'ll usullay point it to a factory function that initializes and\nreturns a Django storage instance because you won\'t want to use globally\nconfigured values for settings such as the S3 bucket name.\n\n``GALLERY_CACHE_STORAGE``\n.........................\n\nDefault: *not defined*\n\nDotted Python path to the Django storage class for resized versions and album\narchives. It must be readable and writable by the application server.\n\nIt behaves like ``GALLERY_PHOTO_STORAGE``.\n\n``GALLERY_PATTERNS``\n....................\n\nDefault: ``()``\n\nTuple of (category name, regular expression) defining how to extract album\ninformation — category, date, name — from the paths of photo files.\n\nThe regular expressions match paths relative to the root of the photo storage.\nThey contain the following captures:\n\n- ``a_name``: album name (mandatory) — to capture several bits, use\n  ``a_name1``, ``a_name2``, etc.\n- ``a_year``, ``a_month``, ``a_day``: album date (mandatory)\n- ``p_year``, ``p_month``, ``p_day``, ``p_hour``, ``p_minute``, ``p_second``:\n  photo date and time (optional)\n\nHere\'s an example, for photos stored with names such as ``2013/01_19_Snow in\nParis/2013-01-19_19-12-43.jpg``::\n\n    GALLERY_PATTERNS = (\n        (\'Photos\',\n            r\'(?P<a_year>\\d{4})/(?P<a_month>\\d{2})_(?P<a_day>\\d{2})_\'\n            r\'(?P<a_name>[^_/]+)/\'\n            r\'(?P<p_year>\\d{4})-(?P<p_month>\\d{2})-(?P<p_day>\\d{2})_\'\n            r\'(?P<p_hour>\\d{2})-(?P<p_minute>\\d{2})-(?P<p_second>\\d{2})\\.jpg\'),\n    )\n\n``GALLERY_IGNORES``\n...................\n\nDefault: ``()``\n\nTuple of regular expressions matching paths within ``GALLERY_PHOTO_STORAGE``.\nFiles matching one of these expressions will be ignored when scanning photos.\n\n``GALLERY_RESIZE``\n..................\n\nDefault: ``gallery.resizers.pillow.resize``\n\nDotted Python path to the callable providing resizing functionality.\n\n``resize(photo, width, height, crop=True)`` receives an instance of the\n``Photo`` model and returns a URL for the resized version. The original image is\nfound at ``photo.image_name`` in the photo storage.\n\nThe default implementation depends on ``Pillow``.\n\n``GALLERY_RESIZE_PRESETS``\n..........................\n\nDefault: ``{}``\n\nDictionary mapping resizing presets to ``(width, height, crop)`` tuples. If\n``crop`` is ``True``, the photo will be cropped and the thumbnail will have\nexactly the requested size. If ``crop`` is ``False``, the thumbnail will be\nsmaller than the requested size in one dimension to preserve the photo\'s\naspect ratio.\n\nThe default templates assume the following values::\n\n    GALLERY_RESIZE_PRESETS = {\n        \'thumb\': (128, 128, True),\n        \'standard\': (768, 768, False),\n    }\n\nYou may double these sizes for better results on high DPI displays.\n\nThe admin expects a ``\'thumb\'`` preset.\n\n``GALLERY_RESIZE_OPTIONS``\n..........................\n\nDefault: ``{}``\n\nDictionary mapping image formats names to dictionaries of options for Pillow\'s\n``save`` method. Pillow\'s documentation describes options for each file format.\n\nThe following a reasonable value for high-quality thumbnails and previews::\n\n    GALLERY_RESIZE_OPTIONS = {\n        \'JPEG\': {\'quality\': 90, \'optimize\': True},\n    }\n\nThe default resizer honors this setting. Other resizers may ignore it.\n\n``GALLERY_TITLE``\n.................\n\nDefault: ``"Gallery"``\n\nTitle of your photo gallery. This is only used by the default templates of the\nindex and year views.\n\n``GALLERY_PREVIEW_COUNT``\n.........................\n\nDefault: ``5``\n\nNumber of thumbnails shown in the preview of each album.\n\nRunning the sample application\n==============================\n\n1.  Make sure Django and Pillow are installed.\n\n2.  Create directories for storing photos and thumbnails::\n\n        $ cd example\n        $ mkdir media\n        $ mkdir media/cache\n        $ mkdir media/photos\n\n3.  Create an album directory, whose name must contain a date, and download\n    images. `Wikipedia\'s featured pictures`_ are a good choice::\n\n        $ mkdir "media/photos/2023_01_01_Featured Pictures"\n        # ... download pictures to this directory...\n\n    .. _Wikipedia\'s featured pictures: https://en.wikipedia.org/wiki/Wikipedia:Featured_pictures\n\n4.  Run the development server::\n\n    $ ./manage.py migrate\n    $ ./manage.py createsuperuser\n    $ ./manage.py runserver\n\n5.  Go to http://localhost:8000/admin/gallery/album/ and log in. Click the\n    "Scan photos" link at the top right, and the "Scan photos" button on the\n    next page. You should see the following messages:\n\n    * Scanning path/to/myks-gallery/example/media/photos\n    * Adding album 2023_01_01_Featured Pictures (Photos) as Featured Pictures\n    * Done (0.01s)\n\n    Go to http://localhost:8000/ and enjoy!\n\n    Since you\'re logged in as an admin user, you can view albums and photos\n    even though you haven\'t defined any access policies yet.\n\nChangelog\n=========\n\n1.0\n---\n\n*Under development*\n\n0.9\n---\n\nThis version makes it possible to customize image resizing, for example to\nintegrate an external service, with the ``GALLERY_RESIZE`` setting. Review\n``gallery.resizers.thumbor.resize`` for an example.\n\nSeveral features designed for storing files in the filesystem are removed:\n\n* The ``--resize`` option of ``django-admin scanphotos`` is removed.\n* Expiration of album archives with the ``GALLERY_ARCHIVE_EXPIRY`` setting is\n  removed. Configure lifecycle for the ``export`` folder on the cloud storage\n  service instead.\n* Fallback to the ``GALLERY_PHOTO_DIR`` and ``GALLERY_CACHE_DIR`` settings,\n  deprecated in version 0.5, is removed.\n* Support for serving files privately from the local filesystem is removed,\n  including the ``GALLERY_SENDFILE_HEADER`` and ``GALLERY_SENDFILE_PREFIX``\n  settings.\n\nIt includes smaller changes too.\n\n* Updated for Django 4.0.\n\n0.8\n---\n\n* Changed photo access policies to always override album access policies, even\n  when "Photos inherit album access policy" is enabled. This makes it possible\n  to restrict access with photo access policies, rather than just extend it.\n* Updated for Django 3.0.\n\n0.7\n---\n\n* Updated for Django 2.0.\n\n0.6\n---\n\n* Added migrations for compatibility with Django 1.9.\n\nTo upgrade an existing project, run: ``django-admin migrate --fake-initial``.\n\n0.5\n---\n\nThis version uses the Django file storage API for all operations on files,\nmaking it possible to use services such as Amazon S3 or Google Cloud Storage\nfor storing photos and thumbnails. It introduces the ``GALLERY_PHOTO_STORAGE``\nand ``GALLERY_CACHE_STORAGE`` settings. They supersede ``GALLERY_PHOTO_DIR``\nand ``GALLERY_CACHE_DIR``.\n\nWhen upgrading to 0.5 or later, you should clear the cache directory.\nPreviously cached thumbnails and exports won\'t be used by this version.\n\nIt includes smaller changes too.\n\n* Switched ordering of albums to always show the most recent albums first.\n* Allowed customizing the number of photos in album previews.\n* Preserved original order of photos in album previews.\n* Added pagination on album preview pages.\n* Changed the hashing schema. This invalides the cache. You should clear it.\n* Fixed collision between zip archives containing photos with the same name.\n\n0.4\n---\n\n* Provided exports of albums as zip archives.\n* Fixed preview of photos affected by batch access policy changes.\n\n0.3\n---\n\n* Support for Python 3 and Django 1.6.\n* Hid public albums by default for logged-in users.\n* Switched the default styles to a responsive design.\n* Added an option to scanphotos to precompute thumbnails.\n* Added an option to scanphotos to resynchronize photo dates.\n* Fixed bugs in photo dates.\n\n0.2\n---\n\n* Made most settings optional for easier deployment.\n* Made "sendfile" optional and used streaming responses as a fallback.\n* Worked around a crash in libjpeg when creating large JPEG previews.\n* Added many tests.\n\n0.1\n---\n\n* Initial public release, with the history from my private repository.\n* Added documentation (README file).\n* Added a sample application.\n',
    'author': 'Aymeric Augustin',
    'author_email': 'aymeric.augustin@m4x.org',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/aaugustin/myks-gallery',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
